#include <algorithm>
#include "curlEasy.h"


const CURLEasy::Global CURLEasy::init;

CURLEasy::CURLEasy() {
	handle = curl_easy_init();
	if (handle == 0) {
		throw Exception();
	}
}

CURLEasy::CURLEasy(const CURLEasy& obj) {
	handle = curl_easy_duphandle(obj.handle);
	if (handle == 0) {
		throw Exception();
	}
}

CURLEasy::~CURLEasy() {
	curl_easy_cleanup(handle);
}

CURLEasy& CURLEasy::operator=(CURLEasy obj) {
	swap(obj);
	return *this;
}

void CURLEasy::swap(CURLEasy& obj) {
	std::swap<CURL*>(handle,obj.handle);
}

void CURLEasy::perform() {
	if (curl_easy_perform(handle) != 0) {
		throw Exception();
	}
}

CURLEasy::Escaped::Escaped(const std::string& str) {
	buffer = curl_easy_escape(0,str.data(),str.size());
	if (buffer == 0) {
		throw Exception();
	}
}

CURLEasy::Escaped::~Escaped() {
	curl_free(buffer);
}

std::string CURLEasy::Escaped::get() const {
	return buffer;
}

CURLEasy::Form::Form() : first(0), last(0) {}

CURLEasy::Form::~Form() {
	curl_formfree(first);
}

struct curl_httppost* CURLEasy::Form::get() const {
	return first;
}

CURLEasy::Global::Global() {
	if (curl_global_init(CURL_GLOBAL_ALL) != CURLE_OK) {
		throw Exception();
	}
}

CURLEasy::Global::~Global() {
	curl_global_cleanup();
}