#ifndef FILE_H
#define FILE_H

#include <string>
#include <stdio.h>


/**
 * Wraps the standard C file function in an exception safe class.
 */
class File {

public:

	/** Exception class thrown by members of this class. */
	class Exception {};


	/**
	 * Opens the given file with the given mode as expected by "fopen".
	 */
	File(const std::string& file, const std::string& mode);

	/**
	 * Closes the file which may fail if there is data to flush. Buf even if
	 * it fails, no exception is thrown as this is to be expected by
	 * destructors.
	 */
	~File();

	/**
	 * Seeks to the given position.
	 */
	void seek(unsigned long position);

	/**
	 * Seeks to the end of file.
	 */
	void seekEOF();

	/**
	 * Returns the current position.
	 */
	unsigned long tell();

	/**
	 * Determines the filesize.
	 */
	size_t size();

	/**
	 * Reads at most "numb" bytes from the file and stores them in "data". The
	 * function returns the number of bytes actually read from the file and
	 * stored in the array.
	 */
	size_t read(char* data, size_t numb);

	/**
	 * Writes exactly "numb" bytes from "data" to the file.
	 */
	void write(const char* data, size_t numb);

private:

	FILE* handle;


	/**
	 * Copying and assigning prohibited.
	 */
	File(const File& obj);
	File& operator=(const File& rhs);

};

#endif