#include "httpRequest.h"
#include "curlEasy.h"


HTTPRequest::HTTPRequest(const Derived<Destination>& dest) : dest(dest) {}

void HTTPRequest::setURL(const std::string& url) {
	this->url = url;
}

const std::string& HTTPRequest::getURL() const {
	return url;
}

void HTTPRequest::setDestination(const Derived<Destination>& dest) {
	this->dest = dest;
}

const Derived<Destination>& HTTPRequest::getDestination() const {
	return dest;
}

std::vector<HTTPRequest::Pair>& HTTPRequest::getListGET() {
	return listGET;
}

const std::vector<HTTPRequest::Pair>& HTTPRequest::getListGET() const {
	return listGET;
}

std::vector<HTTPRequest::Pair>& HTTPRequest::getListPOST() {
	return listPOST;
}

const std::vector<HTTPRequest::Pair>& HTTPRequest::getListPOST() const {
	return listPOST;
}

std::vector<HTTPRequest::Upload>& HTTPRequest::getListUPLOAD() {
	return listUPLOAD;
}

const std::vector<HTTPRequest::Upload>& HTTPRequest::getListUPLOAD() const {
	return listUPLOAD;
}

HTTPRequest::Pair::Pair(const Derived<Source>& name,
	const Derived<Source>& value) :
		name(name), value(value) {}

const Derived<Source>& HTTPRequest::Pair::getName() const {
	return name;
}

const Derived<Source>& HTTPRequest::Pair::getValue() const {
	return value;
}

HTTPRequest::Upload::Upload(const Derived<Source>& name,
	const Derived<Source>& filename,
	const Derived<Source>& data) :
		name(name), filename(filename), data(data) {}

const Derived<Source>& HTTPRequest::Upload::getName() const {
	return name;
}

const Derived<Source>& HTTPRequest::Upload::getFilename() const {
	return filename;
}

const Derived<Source>& HTTPRequest::Upload::getData() const {
	return data;
}

HTTPRequest::SourceURLEncoded::SourceURLEncoded(const Derived<Source>& src) :
	src(src) {}

Derived<Source::Reader> HTTPRequest::SourceURLEncoded::getReader() {
	return Reader(src);
}

HTTPRequest::SourceURLEncoded::Reader::Reader(const Derived<Source>& src) :
	reader(src->getReader()), bufferPos(0) {}

size_t HTTPRequest::SourceURLEncoded::Reader::read(char* data, size_t numb) {
	size_t readTotal = 0;

	while (numb > 0) {

		// Copy as much as possible from internal buffer
		const size_t readSize =
			std::min<size_t>(numb,buffer.size()-bufferPos);
		memcpy(data,buffer.data()+bufferPos,readSize);

		bufferPos += readSize;
		data += readSize;
		numb -= readSize;
		readTotal += readSize;

		// Fill internal buffer if necessary
		if (bufferPos == buffer.size()) {
			
			char srcBuffer[Source::Reader::BUFFER_SIZE];
			const size_t srcSize = reader->read(srcBuffer,BUFFER_SIZE);

			// No more data from source
			if (srcSize == 0) {
				return readTotal;
			}

			// Create new buffer by encoding
			buffer = CURLEasy::Escaped(std::string(srcBuffer,srcSize)).get();
			bufferPos = 0;
		}
	}

	return readTotal;
}

void HTTPRequest::SourceURLEncoded::Reader::rewind() {
	reader->rewind();
	buffer.clear();
	bufferPos = 0;
}

HTTPRequest::SourceURLQuery::SourceURLQuery(
	const std::vector<HTTPRequest::Pair>& src) :
		SourceConcat(concat(src)) {}

std::vector<Derived<Source> > HTTPRequest::SourceURLQuery::concat(
	const std::vector<HTTPRequest::Pair>& src) {
	
	std::vector<Derived<Source> > srcConcat;

	for (unsigned int i = 0; i < src.size(); i++) {
		if (i > 0) {
			srcConcat.push_back(SourceBuffer("&"));
		}

		srcConcat.push_back(SourceURLEncoded(src[i].getName()));
		srcConcat.push_back(SourceBuffer("="));
		srcConcat.push_back(SourceURLEncoded(src[i].getValue()));
	}

	return srcConcat;
}

HTTPRequest::SourceURL::SourceURL(const HTTPRequest& request) :
	SourceConcat(concat(request)) {}

std::vector<Derived<Source> > HTTPRequest::SourceURL::concat(
	const HTTPRequest& request) {
	
	std::vector<Derived<Source> > srcList;
	srcList.push_back(SourceBuffer(request.getURL()));

	if (!request.getListGET().empty()) {
		srcList.push_back(SourceBuffer("?"));
		srcList.push_back(SourceURLQuery(request.getListGET()));
	}

	return srcList;
}

HTTPRequest::SourcePOST::SourcePOST(const HTTPRequest& request) :
	SourceURLQuery(request.getListPOST()) {}

HTTPRequest::Form::Form(const HTTPRequest& request) {
	addPOST(request);
	addUPLOAD(request);
}

void HTTPRequest::Form::addPOST(const HTTPRequest& request) {
	for (unsigned int i = 0; i < request.getListPOST().size(); i++) {
		readerPOST.push_back(
			request.getListPOST()[i].getValue()->getReader());
	}

	for (unsigned int i = 0; i < request.getListPOST().size(); i++) {
		Derived<Source::Reader> readerName(
			request.getListPOST()[i].getName()->getReader());
		std::string name(readerName->toString());

		CURL_EASY_FORM_ADD(*this,
			CURLFORM_COPYNAME,name.data(),
			CURLFORM_NAMELENGTH,static_cast<long>(name.size()),
			CURLFORM_STREAM,(Source::Reader*)(readerPOST[i]),
			CURLFORM_CONTENTSLENGTH,static_cast<long>(readerPOST[i]->size()),
			CURLFORM_END);
	}
}

void HTTPRequest::Form::addUPLOAD(const HTTPRequest& request) {
	for (unsigned int i = 0; i < request.getListUPLOAD().size(); i++) {
		readerUPLOAD.push_back(
			request.getListUPLOAD()[i].getData()->getReader());
	}

	for (unsigned int i = 0; i < request.getListUPLOAD().size(); i++) {
		Derived<Source::Reader> readerName(
			request.getListUPLOAD()[i].getName()->getReader());
		std::string name(readerName->toString());

		Derived<Source::Reader> readerFilename(
			request.getListUPLOAD()[i].getFilename()->getReader());
		std::string filename(readerFilename->toString());
		
		CURL_EASY_FORM_ADD(*this,
			CURLFORM_COPYNAME,name.data(),
			CURLFORM_NAMELENGTH,static_cast<long>(name.size()),
			CURLFORM_FILENAME,filename.c_str(),
			CURLFORM_STREAM,(Source::Reader*)(readerUPLOAD[i]),
			CURLFORM_CONTENTSLENGTH,static_cast<long>(readerUPLOAD[i]->size()),
			CURLFORM_END);
	}
}