#include "source.h"


Source::~Source() {}

Source::Reader::~Reader() {}

size_t Source::Reader::size() {
	char buffer[BUFFER_SIZE];
	size_t readSize;
	size_t readTotal = 0;

	rewind();
	while ((readSize = read(buffer,BUFFER_SIZE)) != 0) {
		readTotal += readSize;
	}
	rewind();

	return readTotal;
}

std::string Source::Reader::toString() {
	char buffer[BUFFER_SIZE];
	size_t readSize;
	std::string str;

	rewind();
	while ((readSize = read(buffer,BUFFER_SIZE)) != 0) {
		str.append(buffer,readSize);
	}
	rewind();

	return str;
}

SourceBuffer::SourceBuffer(const std::string& buffer) : buffer(buffer) {}

Derived<Source::Reader> SourceBuffer::getReader() {
	return Reader(buffer);
}

SourceBuffer::Reader::Reader(const std::string& buffer) :
	buffer(buffer), pos(0) {}

size_t SourceBuffer::Reader::read(char* data, size_t numb) {
	const size_t copy = std::min<size_t>(buffer.size()-pos,numb);
	memcpy(data,buffer.data()+pos,copy);
	pos += copy;

	return copy;
}

void SourceBuffer::Reader::rewind() {
	pos = 0;
}

size_t SourceBuffer::Reader::size() {
	return buffer.size();
}

SourceFile::SourceFile(const std::string& file) : file(file) {}

Derived<Source::Reader> SourceFile::getReader() {
	return Reader(file);
}

SourceFile::Reader::Reader(const std::string& file) :
	handle(new File(file,"rb")) {}

size_t SourceFile::Reader::read(char* data, size_t numb) {
	return handle->read(data,numb);
}

void SourceFile::Reader::rewind() {
	handle->seek(0);
}

size_t SourceFile::Reader::size() {
	return handle->size();
}

SourceConcat::SourceConcat(const std::vector<Derived<Source> >& src) :
	src(src) {}

Derived<Source::Reader> SourceConcat::getReader() {
	return Reader(src);
}

SourceConcat::Reader::Reader(const std::vector<Derived<Source> >& src) :
	current(0) {

	for (unsigned int i = 0; i < src.size(); i++) {
		reader.push_back(src[i]->getReader());
	}
}

size_t SourceConcat::Reader::read(char* data, size_t numb) {
	if (current >= reader.size()) {
		return 0;
	}

	size_t readTotal = 0;

	while (numb > 0) {
		
		const size_t readSize = reader[current]->read(data,numb);

		// When finished with one source goto next.
		if (readSize == 0) {

			current++;

			if (current >= reader.size()) {
				return readTotal;
			}

			reader[current]->rewind();
		}
		
		data += readSize;
		numb -= readSize;
		readTotal += readSize;
	}

	return readTotal;
}

void SourceConcat::Reader::rewind() {
	if (reader.size() > 0) {
		reader[0]->rewind();
	}

	current = 0;
}

size_t SourceConcat::Reader::size() {
	size_t total = 0;

	for (unsigned int i = 0; i < reader.size(); i++) {
		total += reader[i]->size();
	}

	return total;
}