#ifndef THREAD_CPP
#define THREAD_CPP

#include <utility>
#include "thread.h"


Thread::Thread(const Derived<Runner>& runner) {
	Derived<Runner>* runnerPtr = new Derived<Runner>(runner);
	handle = CreateThread(0,0,procWrapper,runnerPtr,0,0);

	if (handle == 0) {
		delete runnerPtr;
		throw Exception();
	}
}

Thread::~Thread() {
	CloseHandle(handle);
}

bool Thread::wait(unsigned long timeout) const {
	return WaitForSingleObject(handle,timeout) == WAIT_TIMEOUT;
}

bool Thread::successful() const {
	DWORD code = EXIT_FAILURE;
	GetExitCodeThread(handle,&code);
	return code == EXIT_SUCCESS;
}

DWORD WINAPI Thread::procWrapper(LPVOID param) {
	DWORD code = EXIT_SUCCESS;
	Derived<Runner>* runnerPtr(
		reinterpret_cast<Derived<Runner>*>(param));

	try {
		(*runnerPtr)->run();
	} catch (...) {
		code = EXIT_FAILURE;
	}

	delete runnerPtr;
	return code;
}

Thread::SharedPtrCounter::SharedPtrCounter() : count(0) {}

void Thread::SharedPtrCounter::operator++() {
	SharedObject<size_t>::Lock lock(count);
	++lock.get();
}

bool Thread::SharedPtrCounter::operator--() {
	SharedObject<size_t>::Lock lock(count);
	return --lock.get() == 0;
}

#endif